import { useJoin, useLocalCameraTrack, useLocalMicrophoneTrack } from 'agora-rtc-react'

import React, { createContext, useContext, useEffect } from 'react'
import type { IMicrophoneAudioTrack, ICameraVideoTrack } from 'agora-rtc-sdk-ng'
import { Box } from '@mui/material'
import { useTheme, type Theme } from '@mui/material/styles'
import MeetingLoading from '../../Components/MeetingLoading/MeetingLoading'
import MeetingGrid from '../../Components/MeetingLayout/MeetingLayout'

interface AgoraContextType {
  localCameraTrack: ICameraVideoTrack | null
  localMicrophoneTrack: IMicrophoneAudioTrack | null
  children: React.ReactNode
}

const AgoraContext = createContext<AgoraContextType | null>(null)

export const AgoraProvider: React.FC<AgoraContextType> = ({
  children,
  localCameraTrack,
  localMicrophoneTrack,
}) => (
  <AgoraContext.Provider value={{ localCameraTrack, localMicrophoneTrack, children }}>
    {children}
  </AgoraContext.Provider>
)

export const useAgoraContext = () => {
  const context = useContext(AgoraContext)
  if (!context) throw new Error('useAgoraContext must be used within an AgoraProvider')
  return context
}

export const AgoraManager = ({ children }: { children: React.ReactNode }) => {
  const theme: Theme = useTheme()

  const {
    data: uid,
    error: joinError,
    isConnected: isJoined,
    isLoading: isJoining,
  } = useJoin({
    appid: '3af4648782de46ddbf90005f7a68f206',
    channel: 'test',
    token: null,
    uid: null,
  })
  const { isLoading: isLoadingCam, localCameraTrack } = useLocalCameraTrack()
  const { isLoading: isLoadingMic, localMicrophoneTrack } = useLocalMicrophoneTrack()

  useEffect(() => {
    return () => {
      localCameraTrack?.close()
      localMicrophoneTrack?.close()
    }
  }, [])

  return (
    <Box
      sx={{
        width: '100%',
        height: '100%',
        backgroundColor: theme.palette.background.default,
        display: 'flex',
      }}
    >
      {isJoining ? (
        <MeetingLoading variant='joining' />
      ) : isLoadingCam || isLoadingMic ? (
        <MeetingLoading variant='loading' />
      ) : isJoined ? (
        <AgoraProvider
          localCameraTrack={localCameraTrack}
          localMicrophoneTrack={localMicrophoneTrack}
        >
          <MeetingGrid />
        </AgoraProvider>
      ) : joinError ? (
        <MeetingLoading variant='error' />
      ) : null}
    </Box>
  )
}

export default AgoraManager
